/**
  **************************************************************************
  * @file     main.c
  * @brief    main program
  **************************************************************************
  *                       Copyright notice & Disclaimer
  *
  * The software Board Support Package (BSP) that is made available to
  * download from Artery official website is the copyrighted work of Artery.
  * Artery authorizes customers to use, copy, and distribute the BSP
  * software and its related documentation for the purpose of design and
  * development in conjunction with Artery microcontrollers. Use of the
  * software is governed by this copyright notice and the following disclaimer.
  *
  * THIS SOFTWARE IS PROVIDED ON "AS IS" BASIS WITHOUT WARRANTIES,
  * GUARANTEES OR REPRESENTATIONS OF ANY KIND. ARTERY EXPRESSLY DISCLAIMS,
  * TO THE FULLEST EXTENT PERMITTED BY LAW, ALL EXPRESS, IMPLIED OR
  * STATUTORY OR OTHER WARRANTIES, GUARANTEES OR REPRESENTATIONS,
  * INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.
  *
  **************************************************************************
  */

#include "at32f403a_407_board.h"
#include "at32f403a_407_clock.h"
#include "flash.h"
#include <stdio.h>
#include <string.h>

uint32_t main_index1 = 0;
uint32_t main_index2 = 0;
uint32_t pvd_write_address = 0;
extern uint32_t save_data[8];

/**
  * @brief  power voltage monitor exint configuration.
  * @param  none
  * @retval none
  */
void pvm_exint_config(void)
{
  exint_init_type exint_init_struct;
  
  /* config the exint line of the power voltage monitor */
  exint_init_struct.line_select = EXINT_LINE_16;
  exint_init_struct.line_enable = TRUE;
  exint_init_struct.line_mode = EXINT_LINE_INTERRUPUT;
  exint_init_struct.line_polarity = EXINT_TRIGGER_RISING_EDGE;
  exint_init(&exint_init_struct);
}

/**
  * @brief  main function.
  * @param  none
  * @retval none
  */
int main(void)
{
  nvic_priority_group_config(NVIC_PRIORITY_GROUP_4);
  system_clock_config();
  at32_board_init();
  
  /* turn on all led,means that code is running */
  at32_led_on(LED2);
  at32_led_on(LED3);
  at32_led_on(LED4);
  delay_ms(500);
  
  /* enable pwc clock */
  crm_periph_clock_enable(CRM_PWC_PERIPH_CLOCK, TRUE);
  
  /* set the threshold voltage to 2.9v */
  pwc_pvm_level_select(PWC_PVM_VOLTAGE_2V9);
  
  /* enable power voltage monitor */
  pwc_power_voltage_monitor_enable(TRUE);
  
  /* config the exint line of the power voltage monitor */
  pvm_exint_config();
 
  /* enable power voltage monitor interrupt */
  nvic_irq_enable(PVM_IRQn, 0, 0);
  
  if(FLASH_SIZE < 256)
  {
    pvd_write_address = 0x08000000 + FLASH_SIZE * 1024 -1024;
  }
  else
  {
    pvd_write_address	= 0x08000000 + FLASH_SIZE * 1024 -2048;
  }
  
  /* the saved data check */
  if(FLASH_SIZE < 256)
  {
    main_index1 = *(volatile uint32_t *)(0x08000000 + FLASH_SIZE * 1024 -1024);
    main_index2 = *(volatile uint32_t *)(0x08000000 + FLASH_SIZE * 1024 -1024 + 28);
  }
  else
  {
    main_index1 = *(volatile uint32_t *)(0x08000000 + FLASH_SIZE * 1024 -2048);
    main_index2 = *(volatile uint32_t *)(0x08000000 + FLASH_SIZE * 1024 -2048 + 28);
  }
  if((main_index1 == save_data[0]) && (main_index2 == save_data[7]))
  {
    /* led2 on and led3 off,means that the data is successfully saved */
    at32_led_on(LED2);
    at32_led_off(LED3);
  }
  else
  {
    /* led2 off and led3 on,means that their was no data being saved or data saved fail */
    at32_led_off(LED2);
    at32_led_on(LED3);
  }
  
  /* erase flash page in advance */
  while(at32_button_press() != USER_BUTTON)
  {
  }
    
  /* unlock the flash controller */
  flash_unlock();
  flash_sector_erase(FLASH_BASE + FLASH_SIZE * 1024 - 2048);
  
  /* lock the flash controller */
  flash_lock();
    
  while(1)
  {
  }
}
